/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms 
of the GNU General Public License as published by the Free Software Foundation; 
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY 
WARRANTY; without even the implied warranty of MERCHANTABILITY or 
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License 
for more details. 

You should have received a copy of the GNU General Public License along with this 
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave, 
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
ldse@hendriklipka.de
*/

package de.hendriklipka.ldse.common;

import java.util.Hashtable;
import java.util.Enumeration;

/**
  Hashtable, die unbenutzte Eintraege nach eine gewissen Zeit rauswirft
*/
public class Cachetable extends Hashtable implements Pingable 
{

  class TimeWrapper 
  {
    private long age;
    private Object object;

    public TimeWrapper( Object object ) 
    {
      this.object = object;
      updateAge();
    }

    private void updateAge() 
    {
      age = System.currentTimeMillis();
    }

    public long getAge() 
    {
      return( System.currentTimeMillis() - age );
    }

    public Object getObject() 
    {
      updateAge();
      return( object );
    }
  }
  
  private long expireTime; 
  private Ping cleaner;

  public Cachetable( long expireTime ) 
  {
    this(expireTime,101,0.75f);
  }
  
  public Cachetable( long expireTime, int initCap ) 
  {
    this(expireTime,initCap,0.75f);
  }
  
  public Cachetable( long expireTime , int initCap, float loadFactor) 
  {
    super(initCap, loadFactor);

    this.expireTime = expireTime;

    cleaner = new Ping(this,(long)(expireTime*0.75),true);
    cleaner.start();
  }

  public synchronized Object put( Object key, Object value ) 
  {
    TimeWrapper tw=new TimeWrapper(value);
    return(super.put(key,tw));
  }

  public synchronized Object get( Object key ) 
  {
    TimeWrapper tw=(TimeWrapper)super.get(key);
    if (null==tw)
    {
      return null;
    }
    return tw.getObject();
  }

  public synchronized void ping() 
  {
    Enumeration e = super.keys();

    while( e.hasMoreElements() ) 
    {
      Object key = e.nextElement();

      TimeWrapper tw = (TimeWrapper) super.get( key );

      if( ( tw != null ) && ( tw.getAge() > expireTime ) ) 
      {
        remove( key );
      }
    }

    System.gc();
  }
}